/*
 Esercizio 8 (file)
 􏰀 È dato un file di testo PEOPLE.TXT che contiene i dati di una serie di persone (non più di 20), una persona per riga
 􏰀 Si vuole realizzare un programma che, una volta letti da file i dati di queste persone, ne estragga l’insieme di persone compatibili con una nuova persona data, salvando il risultato sul file binario PARTNERS.DAT
 • Due persone sono compatibili se sono di sesso diverso e la differenza di età, riferita solo all'anno, non supera i 5 anni
 
 Ogni riga del file contiene, nell'ordine: • cognome (non più di 20 caratteri)
 • un separatore ‘;’
 • nome (non più di 20 caratteri)
 • un separatore ‘;’
 • data di nascita nel formato gg/mm/aaaa • uno e un solo spazio
 • un carattere ('M' o 'F') che indica il sesso
 􏰀 Si definisca opportunamente una struttura dati persona di tipo struct
 
 Realizzare le seguenti funzioni
 - una funzione lettura(...) che, dato il nome del file (ed eventualmente altri parametri se opportuno), legga i dati delle persone dal file e li metta in un array di persona di nome elenco
 – Quanto deve essere grande l’array? – Si mostri a video l'array così costruito
 • una funzione compatibili(...) che, date due persone, restituisca vero solo se le due persone sono compatibili
 – si mostri a video un esempio d'uso della funzione con due persone scelte all'array a vostro piacere
 • main() che invochi lettura per acquisire le persone, poi chieda all’utente i dati di una nuova persona per scrivere infine su file binario l’insieme delle persone compatibili con la persona data
 
 􏰀 Analizzare i “dati”
 • Quanti “dati complessi” sono chiamati in causa? – Persona
 – Data (di nascita)
 • Quali campi? E di che tipo (e dimensione...)?
 􏰀 Seguire un approccio a moduli
 • Sicuramente un modulo separato per la definizione
 delle strutture dati
 • Un modulo per le funzioni chiamate a manipolare queste strutture dati
 􏰀 Lettura da file di testo
 • Individuare il tipo dei campi che vogliamo leggere • ...e i relativi separatori
 
 */

#include <stdio.h>
#include <stdlib.h>

#define NUMEROPERSONE 20
#define DIMCOGNOME 21
#define DIMNOME 21

// Readfield
int readField(char buffer[], char sep, FILE *f);

int readSep(char buffer[], FILE *f)
{
    return readField(buffer, ';', f);
}

// Esempio poi nel main: readSep(persona->nome, fo)
int readField(char buffer[], char sep, FILE *f)
{
	int i = 0;
	char ch = fgetc(f);
	while (ch != sep && ch != 10 && ch != EOF)
	{
		buffer[i] = ch;
		i++;
		ch = fgetc(f);
	}
	buffer[i] = '\0';
}

typedef enum {true, false} Boolean;

typedef struct dataStruct
{
    int giorno, mese, anno;
} Data;

typedef struct personaStruct
{
    char cognome[DIMCOGNOME], nome[DIMNOME], sesso;
    Data nascita;
} Persona;

void lettura(char nomefile[], Persona v[], int* pindice)
{
    Persona x;
    
    *pindice = 0;
    
    FILE *f = fopen(nomefile, "r");
    if (f == NULL)
    {
        printf("Impossibile aprire file di ingresso");
        exit(1);
    }
    
    do {
        readField(x.cognome, ';', f);
        readField(x.nome, ';', f);
        fscanf(f, "%d/%d/%d %c\n", &x.nascita.giorno, &x.nascita.mese, &x.nascita.anno, &x.sesso);

        v[*pindice] = x;
        (*pindice)++;
    } while(!feof(f));
    
    fclose(f);
}

Boolean compatibili(Persona p1, Persona p2)
{
    Boolean compSesso,compAnno;
    
    compSesso = ( p1.sesso != p2.sesso );
    compAnno = abs(p1.nascita.anno - p2.nascita.anno)<=5;
    
    return compSesso && compAnno;
}


int main(void)
{
    Persona elenco[NUMEROPERSONE], utente;
    int indiceElenco = 0, i;
    FILE *fbin;
    
    lettura("PERCORSO_COMPLETO_FILE/PEOPLE.TXT", elenco, &indiceElenco);
    
    printf("\n\nNome e cognome: ");
    scanf("%s",utente.nome);
    scanf("%s",utente.cognome);
    
    printf("data di nascita (gg/mm/aaaa): ");
    scanf("%d/%d/%d", &utente.nascita.giorno, &utente.nascita.mese, &utente.nascita.anno);
    
    /* sopprime il fine linea rimasto sull'input */
   scanf("%*c");
    
    printf("Inserire il sesso (M/F): ");
    scanf("%c%*c",&utente.sesso);
    
    fbin = fopen("PERCORSO_COMPLETO_FILE/PARTNERS.DAT", "wb");
    // Vi ricordo che se il file .dat non esiste, viene creato automaticamente  
    if (fbin==NULL)
    {
        printf("Impossibile aprire file di uscita\n");
        exit(2);
    }
    
    printf("\nCompatibili con %s %s:\n",utente.nome,utente.cognome);
    // Se utilizzate lo start-kit (il file people.txt che vi ho fornito io, il risultato è:
    // asdrubale genoveffa nato(a) il 15/2/1992
    for (i=0; i<indiceElenco; i++)
        if (compatibili(utente,elenco[i]))
        {
            fwrite(&elenco[i], sizeof(Persona), 1, fbin);
            printf("%s %s nato(a) il %d/%d/%d\n", elenco[i].nome, elenco[i].cognome, elenco[i].nascita.giorno, elenco[i].nascita.mese, elenco[i].nascita.anno);
        }
    
    fclose(fbin);
    
    return 0;
}
























